# This file was automatically generated by SWIG
package Ifeffit;
require Exporter;
require DynaLoader;
@ISA = qw(Exporter DynaLoader);
package Ifeffit;
bootstrap Ifeffit;
var_Ifeffit_init();
package Ifeffit;
@EXPORT = qw( );
1;
