C
C fortran header file Ifeffit
        integer   ifeffit,   iffgetecho
        integer   iffgetstr, iffputstr
        integer   iffgetsca, iffputsca
        integer   iffgetarr, iffputarr
        external  ifeffit,   iffgetecho
        external  iffgetstr, iffputstr
        external  iffgetsca, iffputsca
        external  iffgetarr, iffputarr
