c{com.h -*-fortran-*-
       ckeys(1) = '__test'
       chint(1) = 'test function : do not use!'
       ckeys(2) = 'bkg_cl'
       chint(2) = 'remove pre_edge from xmu(E) data with CL data'
       ckeys(3) = 'chi_noise'
       chint(3) = 'calculate noise in a chi(k) spectrum'
       ckeys(4) = 'color'
       chint(4) = 'update the plotting color table'
       ckeys(5) = 'comment'
       chint(5) = 'write a comment into the history buffer'
       ckeys(6) = 'correl'
       chint(6) = 'set two-variable correlation to a Program Variable'
       ckeys(7) = 'cursor'
       chint(7) = 'get the coordinates of the cursor'
       ckeys(8) = 'def'
       chint(8) = 'define the expression and value for a variable'
       ckeys(9) = 'diffkk'
       chint(9) = 'differential Kramers-Kronig transform'
       ckeys(10) = 'echo'
       chint(10) = 'write a message to the screen'
       ckeys(11) = 'erase'
       chint(11) = 'erase a variable'
       ckeys(12) = 'error'
       chint(12) = 'generate the uncertainty in a Program Variable'
       ckeys(13) = 'exit'
       chint(13) = 'exit the program'
       ckeys(14) = 'f1f2'
       chint(14) = 'calculate f1 and f2 from Cromer-Libermann tables'
       ckeys(15) = 'feffit'
       chint(15) = 'execute a feff-based fit to chi(k) data'
       ckeys(16) = 'ff2chi'
       chint(16) = 'convert list of feff paths to chi(k)'
       ckeys(17) = 'fftf'
       chint(17) = 'forward fast fourier transform'
       ckeys(18) = 'fftr'
       chint(18) = 'reverse fast fourier transform'
       ckeys(19) = 'get_path'
       chint(19) = 'convert path to Program Variables'
       ckeys(20) = 'guess'
       chint(20) = 'define a variable as a fitting parameter'
       ckeys(21) = 'history'
       chint(21) = 'set up history buffer and file'
       ckeys(22) = 'linestyle'
       chint(22) = 'update an index in the plotting style table'
       ckeys(23) = 'load'
       chint(23) = 'load a file of commands and execute them'
       ckeys(24) = 'log'
       chint(24) = 'write screen messages to an output file'
       ckeys(25) = 'macro'
       chint(25) = 'define a macro'
       ckeys(26) = 'minimize'
       chint(26) = 'general fitting'
       ckeys(27) = 'newplot'
       chint(27) = 'start a new plot'
       ckeys(28) = 'path'
       chint(28) = 'define a feff scattering path'
       ckeys(29) = 'pause'
       chint(29) = 'wait for user input'
       ckeys(30) = 'plot'
       chint(30) = 'general 2-d plot command'
       ckeys(31) = 'plot_arrow'
       chint(31) = 'put an arrow or line on the plot'
       ckeys(32) = 'plot_marker'
       chint(32) = 'put a single marker on the plot'
       ckeys(33) = 'plot_text'
       chint(33) = 'put a line of text on the plot'
       ckeys(34) = 'pre_edge'
       chint(34) = 'remove pre_edge from xmu(E) data'
       ckeys(35) = 'print'
       chint(35) = 'print command'
       ckeys(36) = 'quit'
       chint(36) = 'quit program'
       ckeys(37) = 'random'
       chint(37) = 'generate arrays of random numbers'
       ckeys(38) = 'read_data'
       chint(38) = 'read a data file'
       ckeys(39) = 'rename'
       chint(39) = 'rename a variable'
       ckeys(40) = 'reset'
       chint(40) = 're-initialize program'
       ckeys(41) = 'restore'
       chint(41) = 'restore session from save file'
       ckeys(42) = 'save'
       chint(42) = 'save session (all program variables and macros)'
       ckeys(43) = 'set'
       chint(43) = 'set the constant value for a variable'
       ckeys(44) = 'show'
       chint(44) = 'show the value of a variable or expression'
       ckeys(45) = 'spline'
       chint(45) = 'determine background spline (autobk algorithm)'
       ckeys(46) = 'sync'
       chint(46) = 'synchronize expressions for arrays'
       ckeys(47) = 'unguess'
       chint(47) = 'convert all *guess* to *set*'
       ckeys(48) = 'window'
       chint(48) = 'create FT window (k->R) without doing FT'
       ckeys(49) = 'write_data'
       chint(49) = 'write a data file'
       ckeys(50) = 'zoom'
       chint(50) = 'choose plotting window with cursor'
c}
