c{atomic.h -*-fortran-*-
       character*2  at_symbol
       integer atomic_z
       external at_symbol, atomic_z
       double precision  at_weight, at_fluor_line
       double precision  at_kedge,  at_l1edge, at_l2edge, at_l3edge
       double precision  at_m1edge, at_m2edge, at_m3edge, at_m4edge
       double precision  at_m5edge, at_n1edge, at_n2edge, at_n3edge
       double precision  at_n4edge, at_n5edge, at_n6edge, at_n7edge
       double precision  at_kwidth, at_l1width, at_l2width, at_l3width

       external  at_weight, at_fluor_line
       external  at_kedge,  at_l1edge, at_l2edge, at_l3edge
       external  at_m1edge, at_m2edge, at_m3edge, at_m4edge
       external  at_m5edge, at_n1edge, at_n2edge, at_n3edge
       external  at_n4edge, at_n5edge, at_n6edge, at_n7edge
       external  at_kwidth, at_l1width, at_l2width, at_l3width
c}
