c Copyright Notice: FEFF6 is copyright protected software and users must
c obtain a license from the University of Washington Office of
c Technology Transfer for its use; see section V of FEFF document.

c Main Authors of FEFF5: please contact us concerning problems.
c A. L. Ankudinov, alex@phys.washington.edu      (206) 543 0435
c S. I. Zabinsky, zabinsky@phys.washington.edu   (206) 543 0435
c J. J. Rehr,     jjr@phys.washington.edu        (206) 543 8593
c R. C. Albers,   rca@nidhug.lanl.gov            (505) 665 0417

c Citations: Please cite at least one of the following articles if 
c FEFF is used in published work: 
c    1) Multiple scattering
c       J.J. Rehr and R.C. Albers, Phys. Rev. B41, 8139 (1990).
c       J.J. Rehr, S.I. Zabinsky and R.C. Albers, 
c          Phys. Rev. Let. 69, 3397 (1992).
c    2) General reference
c       J.J. Rehr, J. Mustre de Leon, S.I. Zabinsky, and R.C. Albers,
c          J. Am. Chem. Soc. 113, 5135 (1991).
c    3) Technical reference
c       J. Mustre de Leon, J.J. Rehr, S.I.  Zabinsky, and R.C. Albers,
c          Phys. Rev. B44, 4146 (1991).


