c{sys.h -*-fortran-*-
c all system and build specific stuff goes here
c to be executed in iff_config.f
       sfile  = 'C:/Program Files/Ifeffit'
       pgdev  = '/gw'
       inifile= 'startup.iff , ifeffit.ini'
       build = '1.0075'//
     $   ' Copyright (c) 2002 Matt Newville, Univ of Chicago'
c}
