c{fitcom.h -*-fortran-*-
c  common blocks for feffit
       implicit none
       include 'const.h'
       include 'fefdat.h'
       include 'chars.h'
       include 'math.h'
       include 'varys.h'
       include 'fft.h'
       include 'data.h'
       include 'bkg.h'
       include 'inout.h'
c fitcom.h}
