bin_dir="/usr/bin"
lib_iff="-L/usr/lib -lifeffit "
lib_plt="-L/usr/local/share/ifeffit/pgplot -lpgplot_iff -lpng -lz -lX11"
lib_f77="-L/usr/lib/gcc/x86_64-redhat-linux/4.1.1 -L/usr/lib/gcc/x86_64-redhat-linux/4.1.1/../../../../lib64 -L/lib/../lib64 -L/usr/lib/../lib64 -lgfortranbegin -lgfortran -lm -lgcc_s "
inc_iff="-I../../src/lib -I/usr/share/ifeffit/config"
